import base64

from flask import Flask, render_template, send_file, request, jsonify

from src.world import World
from src.population import Population

app = Flask(__name__)
world = None
population = None
simulation_frames = {}

cone_number = 30
iterations = 50
population_size = 100
mutation_step_size = 0.1
no_cultural_checked = ""
situational_checked = "checked"
normative_checked = ""
topological_checked = "checked"


@app.route('/', methods=["GET", "POST"])
def main():
    global world
    global population
    global iterations
    global population_size
    global mutation_step_size
    global cone_number
    global no_cultural_checked
    global situational_checked
    global normative_checked
    global topological_checked
    global simulation_frames

    """Entry point; the view for the main page"""

    if 'new_world' in request.form:
        cone_number = int(request.form['cone_number'])
        world = World(cone_number)
        simulation_frames = {}

    if "run_simulation" in request.form:
        simulation_frames = {}
        no_cultural_k = False
        normative_k = False
        topological_k = False
        situational_k = False
        if 'normative_k' in request.form:
            normative_k = True
            normative_checked = "checked"
        else:
            normative_checked = ""
        if 'topological_k' in request.form:
            topological_k = True
            topological_checked = "checked"
        else:
            topological_checked = ""
        if 'situational_k' in request.form:
            situational_k = True
            situational_checked = "checked"
        else:
            situational_checked = ""
        if 'no_cultural_k' in request.form:
            no_cultural_k = True
            no_cultural_checked = "checked"
        else:
            no_cultural_checked = ""
        population_size = int(request.form['population_size'])
        mutation_step_size = float(request.form['mutation_step_size'])
        iterations = int(request.form['iterations'])
        population = Population(population_size, world, default=no_cultural_k,
                                situational=situational_k, normative=normative_k, topological=topological_k,
                                mutation_step_size=mutation_step_size)
        for i in range(iterations + 1):
            simulation_frames[i] = population.plot(False)
            population.new_generation()

    return render_template('main.html', iterations=iterations, population_size=population_size,
                           mutation_step_size=mutation_step_size, cone_number=cone_number,
                           no_cultural_checked=no_cultural_checked, situational_checked=situational_checked,
                           normative_checked=normative_checked, topological_checked=topological_checked)


@app.route('/main.png')
def main_plot():
    global world
    print("world: " + str(world))
    """The view for rendering the scatter chart"""
    if not world:
        world = World(3)
    img = world.get_plot(False)
    return send_file(img, mimetype='image/png', cache_timeout=0)


@app.route('/img/<frame_id>')
def time_plot(frame_id):
    """The view for rendering the scatter chart"""
    # print(simulation_frames)
    # print(frame_id)
    return base64.b64encode(simulation_frames[int(frame_id)].getvalue())


@app.route('/fitness.png')
def fitness_plot():
    print("fitness plot :" + str(population))
    if population:
        img = population.plot_fitness()
        return send_file(img, mimetype='image/png', cache_timeout=0)
    else:
        return '', 204


if __name__ == '__main__':
    app.run()
