from io import BytesIO
from random import random, uniform

from math import sqrt

import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D


class World:
    size_x = 10
    size_y = 10
    gradient_range = (1, 2)
    height_range = (5, 7)

    def __init__(self, cones_n=None):

        # list of tuples (peak_x, peak_y, height, gradient)
        self.cones = []
        if cones_n is None:
            self.cones.append((8, 8, 10, 1))
        else:
            for cone in range(cones_n):
                self.cones.append((random() * self.size_x, random() * self.size_y,
                                   uniform(self.height_range[0], self.height_range[1]),
                                   uniform(self.gradient_range[0], self.gradient_range[1])))

    def height(self, x_position, y_position):
        height_max = 0.0
        if x_position >= 10 or x_position < 0 or y_position >= 10 or y_position < 0:
            return height_max
        for cone in self.cones:
            height_current = cone[2] - sqrt(pow(x_position - cone[0], 2) + pow(y_position - cone[1], 2)) * cone[3]
            if height_current > height_max:
                height_max = height_current
        return height_max

    def get_plot(self, plot_vs_return):

        world_fig = plt.figure()
        world_ax = Axes3D(world_fig)

        x = np.linspace(0, self.size_x, 100)
        y = np.linspace(0, self.size_y, 100)
        x_grid, y_grid = np.meshgrid(x, y)
        z = np.reshape(np.array(list(map(self.height, np.ravel(x_grid), np.ravel(y_grid)))), (100, 100))

        # Plot a basic wire frame.
        world_ax.plot_surface(x_grid, y_grid, z)
        if plot_vs_return:
            plt.show()
            plt.close(world_fig)
        else:
            img = BytesIO()
            plt.savefig(img)
            plt.close(world_fig)
            img.seek(0)
            return img

    def rotate(self):
        pass


def test_world():
    world = World(10)
    world.plot()


if __name__ == '__main__':
    test_world()
